DATA_DIR = "/usr/share"
APP_ID = "org.gnome.tweaks"
PKG_DATA_DIR = "/usr/share/gnome-tweaks"
GSETTINGS_SCHEMA_DIR = "/usr/share/glib-2.0/schemas"
TWEAK_DIR = "/usr/lib/python3.13/site-packages/gtweak/tweaks"
VERSION = "46.1"
LOCALE_DIR = "/usr/share/locale"
